//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-word-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using System.Linq;
using System.Globalization;
using GrapeCity.Documents.Word;

namespace DsWordWeb.Demos
{
    // This example shows how to deal with the 'data source with this name already exists' error.
    public class DataTplFixNoDataSources
    {
        // Code demonstrating the problem:
        GcWordDocument Problem()
        {
            using var oceans = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            var doc = new GcWordDocument();
            doc.Body.Paragraphs.Add("{{ds.name}}");
            // Incorrect: no data sources have been added when the DataTemplate.Process() method is called:
            doc.DataTemplate.Process(CultureInfo.GetCultureInfo("en-US"));
            return doc;
        }

        // Code demonstrating the fix:
        GcWordDocument Fix()
        {
            using var oceans = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            var doc = new GcWordDocument();
            doc.Body.Paragraphs.Add("{{ds.name}}");
            doc.DataTemplate.DataSources.Add("ds", oceans);
            // Correct: the data source have been added prior to calling the DataTemplate.Process() method:
            doc.DataTemplate.Process(CultureInfo.GetCultureInfo("en-US"));
            return doc;
        }

        public GcWordDocument CreateDocx()
        {
            GcWordDocument doc;
            try
            {
                // This fails:
                doc = Problem();
            }
            catch (Exception ex)
            {
                // This works:
                doc = Fix();
                // Insert a brief explanation of the problem and the fix into the generated document:
                doc.Body.Paragraphs.Insert(
                    $"The error \"{ex.Message}\" occurred because the DataTemplate.Process() method was called before adding the data source. " +
                    $"The fix is to add the data sources prior to calling the DataTemplate.Process() method.",
                    doc.Styles[BuiltInStyleId.BlockText],
                    InsertLocation.Start);
            }
            return doc;
        }
    }
}
